from oscars_mgmnt_clms.file_object import ObjectInfo
from oscars_mgmnt_clms.parser.base import BaseParser
from netCDF4 import Dataset

class CLMSGlobalParser(BaseParser):
    """ Parser used to transform CLMS Global Land products to GeoJSON."""

    def __init__(self):
        super().__init__(template='CLMS_GLOBAL.template')

    def convert(self, product_file_info):

        import os
        import time
        import re
        import numpy as np

        # Initialize values dict
        values = {}

        # examples of supported filenames:
        #    c_gls_NDVI300_202204110000_GLOBE_OLCI_V2.0.1.nc
        #    c_gls_BA300-NRT_201901010000_GLOBE_S3_V3.1.1.nc
        fn = os.path.abspath(product_file_info.get_product_path())

        ds = Dataset(fn)

        # filename without extension
        file_name = os.path.basename(fn).rpartition('.')[0]
        file_name_parts = file_name.split('_')

        values['nominaldatetime'] =self.format_fname_date(file_name_parts[3])

        (product_type,separator,product_group) = file_name_parts[2].partition('-')
        if product_group:
            values['productgroupid']=product_group

        version = file_name_parts[-1]
        # major_version = int(version[1])

        tile_id = file_name_parts[-3]

	# collection identifier is constructed from the collection-folder
	# assumes files are stored in 3rd level below collection, ie collection/yyyy/yyyymmdd/file
        path_elem=os.path.dirname(fn).split(os.path.sep)
        collection_dir=path_elem[-3]
        parent_identifier = '_'.join(['clms_global',collection_dir,'netcdf'])

        values['id'] = file_name
        values['title'] = ds.title
        values['parentidentifier'] = parent_identifier

        # TODO extend for SWI DGG product
        values['conformsto'] = 'https://www.opengis.net/def/crs/EPSG/0/4326'

        # netCDF stores center of pixel coordinates, OSCARS stores pixel top left coordinates
        westcenter=np.min(ds.variables['lon'])
        southcenter=np.min(ds.variables['lat'])
        eastcenter=np.max(ds.variables['lon'])
        northcenter=np.max(ds.variables['lat'])
        rasterxsize=len(ds.variables['lon'])
        rasterysize=len(ds.variables['lat'])

        step=(eastcenter-westcenter)/(rasterxsize-1)
        # improve final decimals for PROBA-V/Sentinel-3 vegetation products
        for realstep in [.1, 1./112, 1./336]:
            if (np.round(realstep,10)==np.round(step,10)): 
                step=realstep
                eastcenter=westcenter+((rasterxsize-1)*step)
                southcenter=northcenter-((rasterysize-1)*step)
        halfpixel=step/2.

        # shift half pixel towards from center to top-left of pixel
        values['westbbox'] = np.round(westcenter-halfpixel,10)
        values['southbbox'] = np.round(southcenter-halfpixel,10)
        values['eastbbox'] = np.round(eastcenter+halfpixel,10)
        values['northbbox'] = np.round(northcenter+halfpixel,10)

        values['tileid'] = tile_id
        
        # collection_dir can have 12.5km, 5km, 1km, 300m, 100m or no resolution info
        resolution_regex=re.compile('([\\d.]+)(m|km)')
        regex_result = resolution_regex.match(collection_dir.split('_')[1])

        if regex_result:
            factor = 1000 if regex_result.group(2)[0]=='k' else 1
            values['resolution'] = int(float(regex_result.group(1))*factor)
        else:
            values['resolution'] = None

        values['producttype'] = product_type
        values['productVersion'] = version
        #values['platformShortName'] = ', '.join([x.strip().upper().replace("_","-") for x in ds.platform.split(',')])
        values['beginposition'] = ds.time_coverage_start
        values['endposition'] = ds.time_coverage_end
        values['creationtime'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        values['catingestiondate'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        values['updatetime'] = time.strftime(
            '%Y-%m-%dT%H:%M:%SZ',
            product_file_info.storage.get_time_modified(fn)
        )

        if self.jinja_template is None:
            self.with_template(self.template)

        specific_infos = self.splitted_treatment(product_file_info, values)

        # Generation of the output
        results = {}
        for specific in specific_infos:
            result = self.jinja_template.render(
                dict=specific.values,
                dirpath=specific.product_file_info.get_product_path().rpartition('/')[0],
                products=specific.product_file_info.datas,
                previews=specific.product_file_info.previews,
                metadatas=specific.product_file_info.metadatas,
                related=specific.product_file_info.related,
                platforms=[x.strip().upper().replace("_","-") for x in ds.platform.split(',')]
            )
            results[specific.uid] = result

        return results

    def specific_treatment(self, product_file_info: ObjectInfo, values: dict):
        self.fix_products(product_file_info)
        return values['id']

    @staticmethod
    def fix_products(fileobject):

        import os, re

        for info in list(fileobject.related):
            if info.name.endswith('.nc'):
                title=os.path.basename(info.name).split('_')[2].split('-')[0]
                re_match = re.search("\d", title)
                if re_match:
                    #if needed, use all characters up to first digit (eg NDVI300 -> NDVI)
                    title=title[0:re_match.start()]

                info.title = title
                info.type  = 'application/netcdf'
                fileobject.related.remove(info)
                fileobject.datas.append(info)

    def format_fname_date(self,fname_date):
        return '{}-{}-{}T{}:{}:00Z'.format(fname_date[0:4], fname_date[4:6], fname_date[6:8],
                                           fname_date[8:10], fname_date[10:12])

if __name__ == "__main__":
    import sys
    from oscars_mgmnt_clms.utils.storage import FileSystemInput
    import json

    if len(sys.argv)>1:
        product_path = sys.argv[1]
    else:
        product_path = '/data/MTDA/Copernicus/Land/global/netcdf/ndvi/ndvi_300m_v2_10daily/2022/20220411' \
                       '/c_gls_NDVI300_202204110000_GLOBE_OLCI_V2.0.1.nc'
    product_file_info = ObjectInfo.from_regexp(FileSystemInput(), product_path)
    parser = CLMSGlobalParser()
    results = parser.convert(list(product_file_info.values())[0])
    json_data = json.loads(next(iter(results.values())))
    print(json.dumps(json_data,indent=3))
